---------
--     Anatomy of a SQL query
--     SQL Keywords:
     -- SELECT = indicate the start of a list of columns for the query
     -- INTO = indicates a "make table" query in access (not used in redshift)
     -- FROM <TABLE NAME> = the "starting point" table used by the query
     -- [LEFT/INNER] JOIN <TABLE NAME> = one of more "secondary" tables can be added to the query; requires a list of fields to match up the other table(s) in the query
     -- WHERE = begins a list of filter conditions
     -- GROUP BY = begins list of fields to group by for aggregate queries
     -- QUALIFY = redshift-specific keyword used for "window aggregate" filters. Very useful for effective dating.

-- SQL order of operations:
-- FROM/JOIN, WHERE, SELECT/GROUP BY, HAVING, QUALIFY, ORDER BY

---------------------------------------

-- step 1 - Make Table QUery - creates T00_acad_plan_tbl
-- populated with PS_ACAD_PLAN_TBL filtered to plans active as of the start of a specific term
SELECT PS_ACAD_PLAN_TBL.ACAD_PLAN,
       PS_ACAD_PLAN_TBL.EFFDT,
       PS_ACAD_PLAN_TBL.EFF_STATUS,
       PS_ACAD_PLAN_TBL.DESCR,
       PS_ACAD_PLAN_TBL.ACAD_PLAN_TYPE,
       PS_ACAD_PLAN_TBL.ACAD_PROG,
       PS_ACAD_PLAN_TBL.DEGREE,
       PS_TERM_TBL.STRM
INTO T00_acad_plan_tbl
FROM PS_ACAD_PLAN_TBL, PS_TERM_TBL
WHERE (
          ((PS_ACAD_PLAN_TBL.EFFDT) <= [PS_TERM_TBL]![TERM_BEGIN_DT])
              AND ((PS_TERM_TBL.STRM) = "1148") AND ((PS_TERM_TBL.ACAD_CAREER) = "UGRD")
          );

------------------------------------

-- Q00a_max_effdt - Select query
-- for each plan, get the max effdt (aka most current version of the record as of term begin)
-- here we have only 2 fields, acad_plan and effdt
SELECT T00_acad_plan_tbl.ACAD_PLAN,
       Max(T00_acad_plan_tbl.EFFDT) AS MaxOfEFFDT
FROM T00_acad_plan_tbl
GROUP BY T00_acad_plan_tbl.ACAD_PLAN;

------------------------------------
--Q00b_max_acad_plan_tbl - Make Table Query creates T00b_max acad_plan_tbl
-- now that we know the latest version of each record
-- we can use that info to filter the table we created earlier (T00_acad_plan_tbl)
SELECT T00_acad_plan_tbl.*
INTO [T00b_max acad_plan_tbl]
FROM T00_acad_plan_tbl
    INNER JOIN [Q00a_max effdt]
ON (T00_acad_plan_tbl.ACAD_PLAN = [Q00a_max effdt].ACAD_PLAN)
    AND (T00_acad_plan_tbl.EFFDT = [Q00a_max effdt].MaxOfEFFDT);

--------------------------------------

-- SQL version

-- step 1 - PS_ACAD_PLAN_TBL filtered to plans active as of the start of a specific term
-- this is very similar to the SQL generated by MS Access
-- main difference is the change from an implicit join to an explicit join
-- also does not create an intermediate table, these are not needed with SQL (although they are possible)
-- formatting changes:
--   -add data_student_brz schema name to the beginning of each table name
--   -remove square braces [ ]
--   - change double quotes to single quotes
--   - change [Table]![Field] to Table.Field

SELECT PS_ACAD_PLAN_TBL.ACAD_PLAN,
       PS_ACAD_PLAN_TBL.EFFDT,
       PS_ACAD_PLAN_TBL.EFF_STATUS,
       PS_ACAD_PLAN_TBL.DESCR,
       PS_ACAD_PLAN_TBL.ACAD_PLAN_TYPE,
       PS_ACAD_PLAN_TBL.ACAD_PROG,
       PS_ACAD_PLAN_TBL.DEGREE,
       PS_TERM_TBL.STRM
FROM data_student_brz.PS_ACAD_PLAN_TBL
INNER JOIN data_student_brz.PS_TERM_TBL on PS_ACAD_PLAN_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
    AND PS_TERM_TBL.STRM = '1148' AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
;
--17,312 rows

-------------------------
-- step2 rather than creating an intermediate table, we are going to set up the
-- previous query as a "CTE" (Common Table Expression)
-- A CTE allows us to use the results of a previous query as if it were a table
-- we can name the CTE whatever we want, here we'll use the same name as the intermediate table used in ACcess
-- CTE syntax
-- with <your choice of name> as (<Place query here>)
--
-- select *
-- from <your choice of name>


with T00_acad_plan_tbl as (
    -- this is the first query Q00_acad_plan_tbl
    SELECT PS_ACAD_PLAN_TBL.ACAD_PLAN,
           PS_ACAD_PLAN_TBL.EFFDT,
           PS_ACAD_PLAN_TBL.EFF_STATUS,
           PS_ACAD_PLAN_TBL.DESCR,
           PS_ACAD_PLAN_TBL.ACAD_PLAN_TYPE,
           PS_ACAD_PLAN_TBL.ACAD_PROG,
           PS_ACAD_PLAN_TBL.DEGREE,
           PS_TERM_TBL.STRM
    FROM data_student_brz.PS_ACAD_PLAN_TBL
    INNER JOIN data_student_brz.PS_TERM_TBL on PS_ACAD_PLAN_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
        AND PS_TERM_TBL.STRM = '1148' AND PS_TERM_TBL.ACAD_CAREER = 'UGRD')
-- this is the 2nd query from access, Q00a_max effdt
-- this one is completely unchanged from the access-generated SQL
SELECT T00_acad_plan_tbl.ACAD_PLAN,
       Max(T00_acad_plan_tbl.EFFDT) AS MaxOfEFFDT
FROM T00_acad_plan_tbl
GROUP BY T00_acad_plan_tbl.ACAD_PLAN;

--results = 3598
-- fewer rows since now we have just one row per plan

-------------------

--Step 3
-- we can continue building on to step 2 by "chaining" CTEs together

with T00_acad_plan_tbl as (
    -- this is the first query Q00_acad_plan_tbl
    SELECT PS_ACAD_PLAN_TBL.ACAD_PLAN,
           PS_ACAD_PLAN_TBL.EFFDT,
           PS_ACAD_PLAN_TBL.EFF_STATUS,
           PS_ACAD_PLAN_TBL.DESCR,
           PS_ACAD_PLAN_TBL.ACAD_PLAN_TYPE,
           PS_ACAD_PLAN_TBL.ACAD_PROG,
           PS_ACAD_PLAN_TBL.DEGREE,
           PS_TERM_TBL.STRM
    FROM data_student_brz.PS_ACAD_PLAN_TBL
    INNER JOIN data_student_brz.PS_TERM_TBL on PS_ACAD_PLAN_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
        AND PS_TERM_TBL.STRM = '1148' AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'),
-- you can use multiple CTEs in the same query
-- just add a comma after the first CTE,
-- and repeat the syntax used for a CTE
-- leaving out the "WITH" keyword,
     Q00a_max_effdt as (SELECT T00_acad_plan_tbl.ACAD_PLAN,
                               Max(T00_acad_plan_tbl.EFFDT) AS MaxOfEFFDT
                        FROM T00_acad_plan_tbl
                        GROUP BY T00_acad_plan_tbl.ACAD_PLAN)

-- this is Q00b_max_acad_plan_tbl
SELECT T00_acad_plan_tbl.*
FROM T00_acad_plan_tbl
INNER JOIN Q00a_max_effdt ON T00_acad_plan_tbl.ACAD_PLAN = Q00a_max_effdt.ACAD_PLAN
    AND T00_acad_plan_tbl.EFFDT = Q00a_max_effdt.MaxOfEFFDT;

-- result = 3598 rows

-------------------------------------------------------
-- What we just did was the "Google translate" SQL version of effective dating
-- below is a "native" SQL way to do effective dating


--this is how we would do effective dating with window functions in the legacy ODS
--notice how we needed to wrap the original query inside of another one...
select *
from (
SELECT PS_ACAD_PLAN_TBL.ACAD_PLAN,
       PS_ACAD_PLAN_TBL.EFFDT,
       PS_ACAD_PLAN_TBL.EFF_STATUS,
       PS_ACAD_PLAN_TBL.DESCR,
       PS_ACAD_PLAN_TBL.ACAD_PLAN_TYPE,
       PS_ACAD_PLAN_TBL.ACAD_PROG,
       PS_ACAD_PLAN_TBL.DEGREE,
       PS_TERM_TBL.STRM,
       max(effdt) over (partition by ACAD_PLAN) as mx_effdt
FROM data_student_brz.PS_ACAD_PLAN_TBL
INNER JOIN data_student_brz.PS_TERM_TBL on PS_ACAD_PLAN_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
    AND PS_TERM_TBL.STRM = '1148' AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
) x
WHERE mx_effdt = effdt;

-- but redshift has a shorthand way of doing this, with the QUALIFY keyword:
-- notice that it's just the original Q00 query with one additional line at the end
-- we can rely on this pattern for the remaining tables that we need to effective date
-- just add this line to the end :
-- QUALIFY MAX(<effdt or effseq> OVER(PARTITION BY <fields in group by>) = <effdt or effseq>
-- this will calculate the max effdt or effseq for groups of rows e.g. max effdt per acad_plan and
-- use that to filter the rows

--RAE version of Q00
SELECT PS_ACAD_PLAN_TBL.ACAD_PLAN,
       PS_ACAD_PLAN_TBL.EFFDT,
       PS_ACAD_PLAN_TBL.EFF_STATUS,
       PS_ACAD_PLAN_TBL.DESCR,
       PS_ACAD_PLAN_TBL.ACAD_PLAN_TYPE,
       PS_ACAD_PLAN_TBL.ACAD_PROG,
       PS_ACAD_PLAN_TBL.DEGREE,
       PS_TERM_TBL.STRM
FROM data_student_brz.PS_ACAD_PLAN_TBL
INNER JOIN data_student_brz.PS_TERM_TBL on PS_ACAD_PLAN_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
    AND PS_TERM_TBL.STRM = '1148' AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
QUALIFY max(effdt) over (partition by ACAD_PLAN) = effdt;
;

-- Q01_acad_prog_tbl
SELECT PS_ACAD_PROG_TBL.ACAD_PROG,
       PS_ACAD_PROG_TBL.EFFDT,
       PS_ACAD_PROG_TBL.EFF_STATUS,
       PS_ACAD_PROG_TBL.DESCR,
       PS_ACAD_PROG_TBL.DESCRSHORT,
       PS_TERM_TBL.STRM
INTO T01_acad_prog_tbl
FROM PS_TERM_TBL, PS_ACAD_PROG_TBL
WHERE (((PS_ACAD_PROG_TBL.EFFDT) <= [PS_TERM_TBL]![TERM_BEGIN_DT])
    AND ((PS_TERM_TBL.STRM) = "1148")
    AND ((PS_TERM_TBL.ACAD_CAREER) = "UGRD"));

-- make formatting changes, then add additional line to do the effective dating
-- RAE version of Q01
SELECT PS_ACAD_PROG_TBL.ACAD_PROG,
       PS_ACAD_PROG_TBL.EFFDT,
       PS_ACAD_PROG_TBL.EFF_STATUS,
       PS_ACAD_PROG_TBL.DESCR,
       PS_ACAD_PROG_TBL.DESCRSHORT,
       PS_TERM_TBL.STRM
FROM data_student_brz.PS_TERM_TBL
inner join data_student_brz.PS_ACAD_PROG_TBL on PS_ACAD_PROG_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
    AND PS_TERM_TBL.STRM = '1148'
    AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
QUALIFY max(effdt) over (partition by ACAD_PROG) = effdt;
;
--Do the same for subplans

-- Q02_acad_subplan_tbl
SELECT PS_ACAD_SUBPLN_TBL.ACAD_PLAN,
       PS_ACAD_SUBPLN_TBL.ACAD_SUB_PLAN,
       PS_ACAD_SUBPLN_TBL.EFFDT,
       PS_ACAD_SUBPLN_TBL.EFF_STATUS,
       PS_ACAD_SUBPLN_TBL.ACAD_SUBPLAN_TYPE,
       PS_ACAD_SUBPLN_TBL.DESCR,
       PS_ACAD_SUBPLN_TBL.DESCRSHORT,
       PS_TERM_TBL.STRM
INTO T02_acad_subpln_tbl
FROM PS_ACAD_SUBPLN_TBL, PS_TERM_TBL
WHERE (((PS_ACAD_SUBPLN_TBL.EFFDT) <= PS_TERM_TBL ! TERM_BEGIN_DT)
    And ((PS_TERM_TBL.STRM) = "1148") And ((PS_TERM_TBL.ACAD_CAREER) = "UGRD"));


-- RAE version

SELECT PS_ACAD_SUBPLN_TBL.ACAD_PLAN,
       PS_ACAD_SUBPLN_TBL.ACAD_SUB_PLAN,
       PS_ACAD_SUBPLN_TBL.EFFDT,
       PS_ACAD_SUBPLN_TBL.EFF_STATUS,
       PS_ACAD_SUBPLN_TBL.ACAD_SUBPLAN_TYPE,
       PS_ACAD_SUBPLN_TBL.DESCR,
       PS_ACAD_SUBPLN_TBL.DESCRSHORT,
       PS_TERM_TBL.STRM
FROM data_student_brz.PS_ACAD_SUBPLN_TBL
inner join data_student_brz.PS_TERM_TBL on PS_ACAD_SUBPLN_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
    And data_student_brz.PS_TERM_TBL.STRM = '1148'
    And PS_TERM_TBL.ACAD_CAREER = 'UGRD'
QUALIFY max(effdt) over (partition by ACAD_PLAN, ACAD_SUB_PLAN) = effdt;
-------------------------
-- We'll leave these 3 queries here for now and come back to them in a bit
-------------------------
-- The next query in the job aid is Q03_gather_all_CPPs_information
-- This is a make table query that creates a local table called "T03_gather all CPPS information"
-- This query gets all CPP info for term-activated UGRD students
-- term-activated = PS_STDNT_CAR_TERM WHERE WITHDRAW_CODE = 'NWD'
SELECT PS_STDNT_CAR_TERM.EMPLID,
       PS_STDNT_CAR_TERM.ACAD_CAREER,
       PS_STDNT_CAR_TERM.STRM,
       PS_STDNT_CAR_TERM.WITHDRAW_CODE,
       PS_ACAD_PROG.STDNT_CAR_NBR,
       PS_ACAD_PROG.EFFDT,
       PS_ACAD_PROG.EFFSEQ,
       PS_ACAD_PROG.ACAD_PROG,
       PS_ACAD_PROG.PROG_STATUS,
       PS_ACAD_PROG.PROG_ACTION,
       PS_ACAD_PROG.PROG_REASON,
       PS_ACAD_PLAN.ACAD_PLAN,
       PS_ACAD_PLAN.PLAN_SEQUENCE,
       PS_ACAD_SUBPLAN.ACAD_SUB_PLAN,
       PS_ACAD_PROG.EXP_GRAD_TERM,
       PS_ACAD_PROG.DEGR_CHKOUT_STAT
INTO [T03_gather all CPPS information]
FROM ( (
    ( PS_STDNT_CAR_TERM
    INNER JOIN PS_ACAD_PROG ON (PS_STDNT_CAR_TERM.ACAD_CAREER = PS_ACAD_PROG.ACAD_CAREER) AND (PS_STDNT_CAR_TERM.EMPLID = PS_ACAD_PROG.EMPLID)
    )
    INNER JOIN PS_ACAD_PLAN ON (PS_ACAD_PROG.EFFSEQ = PS_ACAD_PLAN.EFFSEQ)
    AND (PS_ACAD_PROG.EFFDT = PS_ACAD_PLAN.EFFDT)
    AND (PS_ACAD_PROG.STDNT_CAR_NBR = PS_ACAD_PLAN.STDNT_CAR_NBR)
    AND (PS_ACAD_PROG.ACAD_CAREER = PS_ACAD_PLAN.ACAD_CAREER)
    AND (PS_ACAD_PROG.EMPLID = PS_ACAD_PLAN.EMPLID))
    LEFT JOIN PS_ACAD_SUBPLAN ON (PS_ACAD_PLAN.ACAD_PLAN = PS_ACAD_SUBPLAN.ACAD_PLAN)
    AND (PS_ACAD_PLAN.EFFSEQ = PS_ACAD_SUBPLAN.EFFSEQ)
    AND (PS_ACAD_PLAN.EFFDT = PS_ACAD_SUBPLAN.EFFDT)
    AND (PS_ACAD_PLAN.STDNT_CAR_NBR = PS_ACAD_SUBPLAN.STDNT_CAR_NBR)
    AND (PS_ACAD_PLAN.ACAD_CAREER = PS_ACAD_SUBPLAN.ACAD_CAREER)
    AND (PS_ACAD_PLAN.EMPLID = PS_ACAD_SUBPLAN.EMPLID))
    INNER JOIN PS_TERM_TBL
ON PS_STDNT_CAR_TERM.STRM = PS_TERM_TBL.STRM
WHERE (
    ( (PS_STDNT_CAR_TERM.STRM)="1148")
  AND ((PS_STDNT_CAR_TERM.WITHDRAW_CODE)="NWD")
  AND ((PS_ACAD_PROG.EFFDT)<=[PS_TERM_TBL]![TERM_BEGIN_DT])
  AND ((PS_TERM_TBL.ACAD_CAREER)="UGRD"));

-- Since this one is a little more complicated, let's start by first fixing the syntax and
-- removing some redundant parentheses.
-- the order of JOINs in commutative, i.e. changing the order they're written doesn't change the result

-- Q03_gather all CPPS information, v0.1
SELECT PS_STDNT_CAR_TERM.EMPLID,
       PS_STDNT_CAR_TERM.ACAD_CAREER,
       PS_STDNT_CAR_TERM.STRM,
       PS_STDNT_CAR_TERM.WITHDRAW_CODE,
       PS_ACAD_PROG.STDNT_CAR_NBR,
       PS_ACAD_PROG.EFFDT,
       PS_ACAD_PROG.EFFSEQ,
       PS_ACAD_PROG.ACAD_PROG,
       PS_ACAD_PROG.PROG_STATUS,
       PS_ACAD_PROG.PROG_ACTION,
       PS_ACAD_PROG.PROG_REASON,
       PS_ACAD_PLAN.ACAD_PLAN,
       PS_ACAD_PLAN.PLAN_SEQUENCE,
       PS_ACAD_SUBPLAN.ACAD_SUB_PLAN,
       PS_ACAD_PROG.EXP_GRAD_TERM,
       PS_ACAD_PROG.DEGR_CHKOUT_STAT
FROM data_student_brz.PS_STDNT_CAR_TERM
INNER JOIN data_student_brz.PS_TERM_TBL ON PS_STDNT_CAR_TERM.STRM = PS_TERM_TBL.STRM -- moved up
INNER JOIN data_student_brz.PS_ACAD_PROG ON PS_STDNT_CAR_TERM.ACAD_CAREER = PS_ACAD_PROG.ACAD_CAREER
    AND PS_STDNT_CAR_TERM.EMPLID = PS_ACAD_PROG.EMPLID
INNER JOIN data_student_brz.PS_ACAD_PLAN ON PS_ACAD_PROG.EFFSEQ = PS_ACAD_PLAN.EFFSEQ
    AND PS_ACAD_PROG.EFFDT = PS_ACAD_PLAN.EFFDT
    AND PS_ACAD_PROG.STDNT_CAR_NBR = PS_ACAD_PLAN.STDNT_CAR_NBR
    AND PS_ACAD_PROG.ACAD_CAREER = PS_ACAD_PLAN.ACAD_CAREER
    AND PS_ACAD_PROG.EMPLID = PS_ACAD_PLAN.EMPLID
LEFT JOIN data_student_brz.PS_ACAD_SUBPLAN ON PS_ACAD_PLAN.ACAD_PLAN = PS_ACAD_SUBPLAN.ACAD_PLAN
    AND PS_ACAD_PLAN.EFFSEQ = PS_ACAD_SUBPLAN.EFFSEQ
    AND PS_ACAD_PLAN.EFFDT = PS_ACAD_SUBPLAN.EFFDT
    AND PS_ACAD_PLAN.STDNT_CAR_NBR = PS_ACAD_SUBPLAN.STDNT_CAR_NBR
    AND PS_ACAD_PLAN.ACAD_CAREER = PS_ACAD_SUBPLAN.ACAD_CAREER
    AND PS_ACAD_PLAN.EMPLID = PS_ACAD_SUBPLAN.EMPLID
WHERE PS_STDNT_CAR_TERM.STRM = '1148'
  AND PS_STDNT_CAR_TERM.WITHDRAW_CODE = 'NWD'
  --below line is an implicit join condition that should be moved into the join clause of ps_acad_prog
  AND PS_ACAD_PROG.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
  AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
;

--returns 628,591 rows
-- Now we have all CPP stacks for term-activated students,
-- but we still need to do some effective dating, since right now we have
-- every version of each CPP stack that existed as of the start of 1148 term
-- we only care about the most recent version for each emplid/acad_career/stdnt_car_nbr combination
-- MS Access handles this much the same way as the first example with PS_ACAD_PLAN_TBL
-- Here are "Query Q03a_max effdt CPPS data" and "Q03b max effseq CPPs data"


-- get max effdt per emplid/acad_career/stdnt_car_nbr combination
SELECT [T03_gather all CPPS information].EMPLID,
       [T03_gather all CPPS information].ACAD_CAREER,
    [T03_gather all CPPS information].STDNT_CAR_NBR,
    Max ([T03_gather all CPPS information].EFFDT) AS MaxOfEFFDT
FROM [T03_gather all CPPS information]
GROUP BY [T03_gather all CPPS information].EMPLID,
    [T03_gather all CPPS information].ACAD_CAREER,
    [T03_gather all CPPS information].STDNT_CAR_NBR;

-- get max effseq per emplid/acad_career/stdnt_car_nbr/EFFDT combination
SELECT [T03_gather all CPPS information].EMPLID,
       [T03_gather all CPPS information].ACAD_CAREER,
    [T03_gather all CPPS information].STDNT_CAR_NBR,
    [T03_gather all CPPS information].EFFDT,
    Max ([T03_gather all CPPS information].EFFSEQ) AS MaxOfEFFSEQ
FROM [T03_gather all CPPS information]
    INNER JOIN [Q03a_max effdt CPPS data]
ON ([T03_gather all CPPS information].EMPLID = [Q03a_max effdt CPPS data].EMPLID)
    AND ([T03_gather all CPPS information].ACAD_CAREER = [Q03a_max effdt CPPS data].ACAD_CAREER)
    AND ([T03_gather all CPPS information].STDNT_CAR_NBR = [Q03a_max effdt CPPS data].STDNT_CAR_NBR)
    AND ([T03_gather all CPPS information].EFFDT = [Q03a_max effdt CPPS data].MaxOfEFFDT)
GROUP BY [T03_gather all CPPS information].EMPLID,
    [T03_gather all CPPS information].ACAD_CAREER,
    [T03_gather all CPPS information].STDNT_CAR_NBR,
    [T03_gather all CPPS information].EFFDT;


-- We can use the same pattern we used in the PS_ACAD_PLAN_TBL example
-- to replace both these queries
-- with just a few lines added to the end of Q03
-- since we need to both effective date and effective sequence this dataset
-- we'll need two different conditions in the QUALIFY clause
-- one of the lines is an additional filter found in Q03c, PROG_STATUS='AC'
-- which needs to be applied AFTER or at the same time as effdt-ing and effseq-ing
-- (so it can't go in the WHERE clause)

-- Q03_gather all CPPS information, v0.2
SELECT PS_STDNT_CAR_TERM.EMPLID,
       PS_STDNT_CAR_TERM.ACAD_CAREER,
       PS_STDNT_CAR_TERM.STRM,
       PS_STDNT_CAR_TERM.WITHDRAW_CODE,
       PS_ACAD_PROG.STDNT_CAR_NBR,
       PS_ACAD_PROG.EFFDT,
       PS_ACAD_PROG.EFFSEQ,
       PS_ACAD_PROG.ACAD_PROG,
       PS_ACAD_PROG.PROG_STATUS,
       PS_ACAD_PROG.PROG_ACTION,
       PS_ACAD_PROG.PROG_REASON,
       PS_ACAD_PLAN.ACAD_PLAN,
       PS_ACAD_PLAN.PLAN_SEQUENCE,
       PS_ACAD_SUBPLAN.ACAD_SUB_PLAN,
       PS_ACAD_PROG.EXP_GRAD_TERM,
       PS_ACAD_PROG.DEGR_CHKOUT_STAT
FROM data_student_brz.PS_STDNT_CAR_TERM
INNER JOIN data_student_brz.PS_TERM_TBL ON PS_STDNT_CAR_TERM.STRM = PS_TERM_TBL.STRM -- moved up
INNER JOIN data_student_brz.PS_ACAD_PROG ON PS_STDNT_CAR_TERM.ACAD_CAREER = PS_ACAD_PROG.ACAD_CAREER
    AND PS_STDNT_CAR_TERM.EMPLID = PS_ACAD_PROG.EMPLID
    AND PS_ACAD_PROG.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT --moved from where clause
INNER JOIN data_student_brz.PS_ACAD_PLAN ON PS_ACAD_PROG.EFFSEQ = PS_ACAD_PLAN.EFFSEQ
    AND PS_ACAD_PROG.EFFDT = PS_ACAD_PLAN.EFFDT
    AND PS_ACAD_PROG.STDNT_CAR_NBR = PS_ACAD_PLAN.STDNT_CAR_NBR
    AND PS_ACAD_PROG.ACAD_CAREER = PS_ACAD_PLAN.ACAD_CAREER
    AND PS_ACAD_PROG.EMPLID = PS_ACAD_PLAN.EMPLID
LEFT JOIN data_student_brz.PS_ACAD_SUBPLAN ON PS_ACAD_PLAN.ACAD_PLAN = PS_ACAD_SUBPLAN.ACAD_PLAN
    AND PS_ACAD_PLAN.EFFSEQ = PS_ACAD_SUBPLAN.EFFSEQ
    AND PS_ACAD_PLAN.EFFDT = PS_ACAD_SUBPLAN.EFFDT
    AND PS_ACAD_PLAN.STDNT_CAR_NBR = PS_ACAD_SUBPLAN.STDNT_CAR_NBR
    AND PS_ACAD_PLAN.ACAD_CAREER = PS_ACAD_SUBPLAN.ACAD_CAREER
    AND PS_ACAD_PLAN.EMPLID = PS_ACAD_SUBPLAN.EMPLID
WHERE PS_STDNT_CAR_TERM.STRM = '1148'
  AND PS_STDNT_CAR_TERM.WITHDRAW_CODE = 'NWD'
  AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
QUALIFY max(PS_ACAD_PROG.effdt)
        over (PARTITION BY PS_STDNT_CAR_TERM.EMPLID,PS_STDNT_CAR_TERM.ACAD_CAREER,PS_ACAD_PROG.STDNT_CAR_NBR)
    = PS_ACAD_PROG.EFFDT
    AND max(PS_ACAD_PROG.effseq)
        over (PARTITION BY PS_STDNT_CAR_TERM.EMPLID,PS_STDNT_CAR_TERM.ACAD_CAREER,PS_ACAD_PROG.STDNT_CAR_NBR, PS_ACAD_PROG.EFFDT)
    = PS_ACAD_PROG.EFFSEQ
    AND prog_status = 'AC' -- filter found in query Q03c
;
--returns 94,858 rows
-- this query returns the same result as Q03c
-- EFFDT'd CPPS for term-activated students

-- All that remains is to put it all together
-- by combining our last query with fields from the setup tables
-- we worked on in part 1

-- To start let's start building a query with all 4 of the previous queries set up as CTEs:
-- to make it easier to read, let's shorten the names to just the Table #

with T01b as (SELECT PS_ACAD_PROG_TBL.ACAD_PROG,
                     PS_ACAD_PROG_TBL.EFFDT,
                     PS_ACAD_PROG_TBL.EFF_STATUS,
                     PS_ACAD_PROG_TBL.DESCR,
                     PS_ACAD_PROG_TBL.DESCRSHORT,
                     PS_TERM_TBL.STRM
              FROM data_student_brz.PS_TERM_TBL
              inner join data_student_brz.PS_ACAD_PROG_TBL on PS_ACAD_PROG_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
                  AND PS_TERM_TBL.STRM = '1148'
                  AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
              QUALIFY ROW_NUMBER() over (partition by ACAD_PROG order by EFFDT DESC) = 1),
     T00b as (SELECT PS_ACAD_PLAN_TBL.ACAD_PLAN,
                     PS_ACAD_PLAN_TBL.EFFDT,
                     PS_ACAD_PLAN_TBL.EFF_STATUS,
                     PS_ACAD_PLAN_TBL.DESCR,
                     PS_ACAD_PLAN_TBL.ACAD_PLAN_TYPE,
                     PS_ACAD_PLAN_TBL.ACAD_PROG,
                     PS_ACAD_PLAN_TBL.DEGREE,
                     PS_TERM_TBL.STRM
              FROM data_student_brz.PS_ACAD_PLAN_TBL
              INNER JOIN data_student_brz.PS_TERM_TBL on PS_ACAD_PLAN_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
                  AND PS_TERM_TBL.STRM = '1148' AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
              QUALIFY ROW_NUMBER() over (partition by ACAD_PLAN order by EFFDT DESC) = 1),
     T02b as (SELECT PS_ACAD_SUBPLN_TBL.ACAD_PLAN,
                     PS_ACAD_SUBPLN_TBL.ACAD_SUB_PLAN,
                     PS_ACAD_SUBPLN_TBL.EFFDT,
                     PS_ACAD_SUBPLN_TBL.EFF_STATUS,
                     PS_ACAD_SUBPLN_TBL.ACAD_SUBPLAN_TYPE,
                     PS_ACAD_SUBPLN_TBL.DESCR,
                     PS_ACAD_SUBPLN_TBL.DESCRSHORT,
                     PS_TERM_TBL.STRM
              FROM data_student_brz.PS_ACAD_SUBPLN_TBL
              inner join data_student_brz.PS_TERM_TBL
                  on PS_ACAD_SUBPLN_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
                  And data_student_brz.PS_TERM_TBL.STRM = '1148'
                  And PS_TERM_TBL.ACAD_CAREER = 'UGRD'
              QUALIFY ROW_NUMBER() over (partition by ACAD_PLAN, ACAD_SUB_PLAN order by EFFDT DESC) =
                      1),
     T03c as (SELECT PS_STDNT_CAR_TERM.EMPLID,
                     PS_STDNT_CAR_TERM.ACAD_CAREER,
                     PS_STDNT_CAR_TERM.STRM,
                     PS_STDNT_CAR_TERM.WITHDRAW_CODE,
                     PS_ACAD_PROG.STDNT_CAR_NBR,
                     PS_ACAD_PROG.EFFDT,
                     PS_ACAD_PROG.EFFSEQ,
                     PS_ACAD_PROG.ACAD_PROG,
                     PS_ACAD_PROG.PROG_STATUS,
                     PS_ACAD_PROG.PROG_ACTION,
                     PS_ACAD_PROG.PROG_REASON,
                     PS_ACAD_PLAN.ACAD_PLAN,
                     PS_ACAD_PLAN.PLAN_SEQUENCE,
                     PS_ACAD_SUBPLAN.ACAD_SUB_PLAN,
                     PS_ACAD_PROG.EXP_GRAD_TERM,
                     PS_ACAD_PROG.DEGR_CHKOUT_STAT
              FROM data_student_brz.PS_STDNT_CAR_TERM
              INNER JOIN data_student_brz.PS_TERM_TBL
                  ON PS_STDNT_CAR_TERM.STRM = PS_TERM_TBL.STRM -- moved up
              INNER JOIN data_student_brz.PS_ACAD_PROG
                  ON PS_STDNT_CAR_TERM.ACAD_CAREER = PS_ACAD_PROG.ACAD_CAREER
                  AND PS_STDNT_CAR_TERM.EMPLID = PS_ACAD_PROG.EMPLID
                  AND PS_ACAD_PROG.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT --moved from where clause
              INNER JOIN data_student_brz.PS_ACAD_PLAN ON PS_ACAD_PROG.EFFSEQ = PS_ACAD_PLAN.EFFSEQ
                  AND PS_ACAD_PROG.EFFDT = PS_ACAD_PLAN.EFFDT
                  AND PS_ACAD_PROG.STDNT_CAR_NBR = PS_ACAD_PLAN.STDNT_CAR_NBR
                  AND PS_ACAD_PROG.ACAD_CAREER = PS_ACAD_PLAN.ACAD_CAREER
                  AND PS_ACAD_PROG.EMPLID = PS_ACAD_PLAN.EMPLID
              LEFT JOIN data_student_brz.PS_ACAD_SUBPLAN
                  ON PS_ACAD_PLAN.ACAD_PLAN = PS_ACAD_SUBPLAN.ACAD_PLAN
                  AND PS_ACAD_PLAN.EFFSEQ = PS_ACAD_SUBPLAN.EFFSEQ
                  AND PS_ACAD_PLAN.EFFDT = PS_ACAD_SUBPLAN.EFFDT
                  AND PS_ACAD_PLAN.STDNT_CAR_NBR = PS_ACAD_SUBPLAN.STDNT_CAR_NBR
                  AND PS_ACAD_PLAN.ACAD_CAREER = PS_ACAD_SUBPLAN.ACAD_CAREER
                  AND PS_ACAD_PLAN.EMPLID = PS_ACAD_SUBPLAN.EMPLID
              WHERE PS_STDNT_CAR_TERM.STRM = '1148'
                AND PS_STDNT_CAR_TERM.WITHDRAW_CODE = 'NWD'
                AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
              QUALIFY max(PS_ACAD_PROG.effdt)
                      over (PARTITION BY PS_STDNT_CAR_TERM.EMPLID,PS_STDNT_CAR_TERM.ACAD_CAREER,PS_ACAD_PROG.STDNT_CAR_NBR)
                  = PS_ACAD_PROG.EFFDT
                  AND max(PS_ACAD_PROG.effseq)
                      over (PARTITION BY PS_STDNT_CAR_TERM.EMPLID,PS_STDNT_CAR_TERM.ACAD_CAREER,PS_ACAD_PROG.STDNT_CAR_NBR, PS_ACAD_PROG.EFFDT)
                  = PS_ACAD_PROG.EFFSEQ
                  AND prog_status = 'AC' -- filter found in query Q03c
     )
     --Once we finish converting Q04, it will go here
    <final query goes here>
-- Now we have our first 4 queries set up as CTEs and we can use them as if they were tables in the final query.

;
--------------------------------------
-- Preparing the final query Q04

--This is the final query in the database, Q04_CPPS data with setup tbl information
-- this one looks worse than it is because of the very long table name
-- when we reformat it, we'll replace the long names with the versions from our CTEs above
-- makes table "T04_CPPS data with setup tbl information"
SELECT [T03c_max career program plan and subplan information].EMPLID,
    [T03c_max career program plan and subplan information].ACAD_CAREER,
    [T03c_max career program plan and subplan information].STRM,
    [T03c_max career program plan and subplan information].WITHDRAW_CODE,
    [T03c_max career program plan and subplan information].STDNT_CAR_NBR,
    [T03c_max career program plan and subplan information].EFFDT,
    [T03c_max career program plan and subplan information].EFFSEQ,
    [T03c_max career program plan and subplan information].ACAD_PROG,
    [T01b_max acad_prog_tbl].DESCR AS PROG_DESCR,
    [T03c_max career program plan and subplan information].PROG_STATUS,
    [T03c_max career program plan and subplan information].PROG_ACTION,
    [T03c_max career program plan and subplan information].PROG_REASON,
    [T03c_max career program plan and subplan information].ACAD_PLAN,
    [T00b_max acad_plan_tbl].DESCR AS PLAN_DESC,
    [T00b_max acad_plan_tbl].ACAD_PLAN_TYPE,
    [T03c_max career program plan and subplan information].PLAN_SEQUENCE,
    [T03c_max career program plan and subplan information].ACAD_SUB_PLAN,
    [T02b_max acad_subpln_tbl].DESCR AS SUBPLAN_DESC,
    [T03c_max career program plan and subplan information].EXP_GRAD_TERM,
    [T03c_max career program plan and subplan information].DEGR_CHKOUT_STAT
INTO [T04_CPPS data
with setup tbl information]
FROM (([T03c_max career program plan and subplan information]
LEFT JOIN [T01b_max acad_prog_tbl] ON [T03c_max career program plan and subplan information].ACAD_PROG = [T01b_max acad_prog_tbl].ACAD_PROG)
LEFT JOIN [T00b_max acad_plan_tbl] ON [T03c_max career program plan and subplan information].ACAD_PLAN = [T00b_max acad_plan_tbl].ACAD_PLAN)
LEFT JOIN [T02b_max acad_subpln_tbl] ON ([T03c_max career program plan and subplan information].ACAD_PLAN = [T02b_max acad_subpln_tbl].ACAD_PLAN)
AND ([T03c_max career program plan and subplan information].ACAD_SUB_PLAN = [T02b_max acad_subpln_tbl].ACAD_SUB_PLAN);

-- after cleaning it up:
SELECT T03c.EMPLID,
       T03c.ACAD_CAREER,
       T03c.STRM,
       T03c.WITHDRAW_CODE,
       T03c.STDNT_CAR_NBR,
       T03c.EFFDT,
       T03c.EFFSEQ,
       T03c.ACAD_PROG,
       T01b.DESCR AS PROG_DESCR,
       T03c.PROG_STATUS,
       T03c.PROG_ACTION,
       T03c.PROG_REASON,
       T03c.ACAD_PLAN,
       T00b.DESCR AS PLAN_DESC,
       T00b.ACAD_PLAN_TYPE,
       T03c.PLAN_SEQUENCE,
       T03c.ACAD_SUB_PLAN,
       T02b.DESCR AS SUBPLAN_DESC,
       T03c.EXP_GRAD_TERM,
       T03c.DEGR_CHKOUT_STAT
FROM T03c
LEFT JOIN T01b ON T03c.ACAD_PROG = T01b.ACAD_PROG
LEFT JOIN T00b ON T03c.ACAD_PLAN = T00b.ACAD_PLAN
LEFT JOIN T02b ON T03c.ACAD_PLAN = T02b.ACAD_PLAN
    AND T03c.ACAD_SUB_PLAN = T02b.ACAD_SUB_PLAN;

-- much better....
-- before we can run this though, we need to add it to the end of our 4 CTEs
-- that we set up earlier
-- so here's the entire query all together

with T01b as (SELECT PS_ACAD_PROG_TBL.ACAD_PROG,
                     PS_ACAD_PROG_TBL.EFFDT,
                     PS_ACAD_PROG_TBL.EFF_STATUS,
                     PS_ACAD_PROG_TBL.DESCR,
                     PS_ACAD_PROG_TBL.DESCRSHORT,
                     PS_TERM_TBL.STRM
              FROM data_student_brz.PS_TERM_TBL
              inner join data_student_brz.PS_ACAD_PROG_TBL on PS_ACAD_PROG_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
                  AND PS_TERM_TBL.STRM = '1148'
                  AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
              QUALIFY ROW_NUMBER() over (partition by ACAD_PROG order by EFFDT DESC) = 1),
     T00b as (SELECT PS_ACAD_PLAN_TBL.ACAD_PLAN,
                     PS_ACAD_PLAN_TBL.EFFDT,
                     PS_ACAD_PLAN_TBL.EFF_STATUS,
                     PS_ACAD_PLAN_TBL.DESCR,
                     PS_ACAD_PLAN_TBL.ACAD_PLAN_TYPE,
                     PS_ACAD_PLAN_TBL.ACAD_PROG,
                     PS_ACAD_PLAN_TBL.DEGREE,
                     PS_TERM_TBL.STRM
              FROM data_student_brz.PS_ACAD_PLAN_TBL
              INNER JOIN data_student_brz.PS_TERM_TBL on PS_ACAD_PLAN_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
                  AND PS_TERM_TBL.STRM = '1148' AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
              QUALIFY ROW_NUMBER() over (partition by ACAD_PLAN order by EFFDT DESC) = 1),
     T02b as (SELECT PS_ACAD_SUBPLN_TBL.ACAD_PLAN,
                     PS_ACAD_SUBPLN_TBL.ACAD_SUB_PLAN,
                     PS_ACAD_SUBPLN_TBL.EFFDT,
                     PS_ACAD_SUBPLN_TBL.EFF_STATUS,
                     PS_ACAD_SUBPLN_TBL.ACAD_SUBPLAN_TYPE,
                     PS_ACAD_SUBPLN_TBL.DESCR,
                     PS_ACAD_SUBPLN_TBL.DESCRSHORT,
                     PS_TERM_TBL.STRM
              FROM data_student_brz.PS_ACAD_SUBPLN_TBL
              inner join data_student_brz.PS_TERM_TBL
                  on PS_ACAD_SUBPLN_TBL.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT
                  And data_student_brz.PS_TERM_TBL.STRM = '1148'
                  And PS_TERM_TBL.ACAD_CAREER = 'UGRD'
              QUALIFY ROW_NUMBER() over (partition by ACAD_PLAN, ACAD_SUB_PLAN order by EFFDT DESC) =
                      1),
     T03c as (SELECT PS_STDNT_CAR_TERM.EMPLID,
                     PS_STDNT_CAR_TERM.ACAD_CAREER,
                     PS_STDNT_CAR_TERM.STRM,
                     PS_STDNT_CAR_TERM.WITHDRAW_CODE,
                     PS_ACAD_PROG.STDNT_CAR_NBR,
                     PS_ACAD_PROG.EFFDT,
                     PS_ACAD_PROG.EFFSEQ,
                     PS_ACAD_PROG.ACAD_PROG,
                     PS_ACAD_PROG.PROG_STATUS,
                     PS_ACAD_PROG.PROG_ACTION,
                     PS_ACAD_PROG.PROG_REASON,
                     PS_ACAD_PLAN.ACAD_PLAN,
                     PS_ACAD_PLAN.PLAN_SEQUENCE,
                     PS_ACAD_SUBPLAN.ACAD_SUB_PLAN,
                     PS_ACAD_PROG.EXP_GRAD_TERM,
                     PS_ACAD_PROG.DEGR_CHKOUT_STAT
              FROM data_student_brz.PS_STDNT_CAR_TERM
              INNER JOIN data_student_brz.PS_TERM_TBL
                  ON PS_STDNT_CAR_TERM.STRM = PS_TERM_TBL.STRM -- moved up
              INNER JOIN data_student_brz.PS_ACAD_PROG
                  ON PS_STDNT_CAR_TERM.ACAD_CAREER = PS_ACAD_PROG.ACAD_CAREER
                  AND PS_STDNT_CAR_TERM.EMPLID = PS_ACAD_PROG.EMPLID
                  AND PS_ACAD_PROG.EFFDT <= PS_TERM_TBL.TERM_BEGIN_DT --moved from where clause
              INNER JOIN data_student_brz.PS_ACAD_PLAN ON PS_ACAD_PROG.EFFSEQ = PS_ACAD_PLAN.EFFSEQ
                  AND PS_ACAD_PROG.EFFDT = PS_ACAD_PLAN.EFFDT
                  AND PS_ACAD_PROG.STDNT_CAR_NBR = PS_ACAD_PLAN.STDNT_CAR_NBR
                  AND PS_ACAD_PROG.ACAD_CAREER = PS_ACAD_PLAN.ACAD_CAREER
                  AND PS_ACAD_PROG.EMPLID = PS_ACAD_PLAN.EMPLID
              LEFT JOIN data_student_brz.PS_ACAD_SUBPLAN
                  ON PS_ACAD_PLAN.ACAD_PLAN = PS_ACAD_SUBPLAN.ACAD_PLAN
                  AND PS_ACAD_PLAN.EFFSEQ = PS_ACAD_SUBPLAN.EFFSEQ
                  AND PS_ACAD_PLAN.EFFDT = PS_ACAD_SUBPLAN.EFFDT
                  AND PS_ACAD_PLAN.STDNT_CAR_NBR = PS_ACAD_SUBPLAN.STDNT_CAR_NBR
                  AND PS_ACAD_PLAN.ACAD_CAREER = PS_ACAD_SUBPLAN.ACAD_CAREER
                  AND PS_ACAD_PLAN.EMPLID = PS_ACAD_SUBPLAN.EMPLID
              WHERE PS_STDNT_CAR_TERM.STRM = '1148'
                AND PS_STDNT_CAR_TERM.WITHDRAW_CODE = 'NWD'
                AND PS_TERM_TBL.ACAD_CAREER = 'UGRD'
              QUALIFY max(PS_ACAD_PROG.effdt)
                      over (PARTITION BY PS_STDNT_CAR_TERM.EMPLID,PS_STDNT_CAR_TERM.ACAD_CAREER,PS_ACAD_PROG.STDNT_CAR_NBR)
                  = PS_ACAD_PROG.EFFDT
                  AND max(PS_ACAD_PROG.effseq)
                      over (PARTITION BY PS_STDNT_CAR_TERM.EMPLID,PS_STDNT_CAR_TERM.ACAD_CAREER,PS_ACAD_PROG.STDNT_CAR_NBR, PS_ACAD_PROG.EFFDT)
                  = PS_ACAD_PROG.EFFSEQ
                  AND prog_status = 'AC' -- filter found in query Q03c
              )
--Q04
SELECT T03c.EMPLID,
       T03c.ACAD_CAREER,
       T03c.STRM,
       T03c.WITHDRAW_CODE,
       T03c.STDNT_CAR_NBR,
       T03c.EFFDT,
       T03c.EFFSEQ,
       T03c.ACAD_PROG,
       T01b.DESCR AS PROG_DESCR,
       T03c.PROG_STATUS,
       T03c.PROG_ACTION,
       T03c.PROG_REASON,
       T03c.ACAD_PLAN,
       T00b.DESCR AS PLAN_DESC,
       T00b.ACAD_PLAN_TYPE,
       T03c.PLAN_SEQUENCE,
       T03c.ACAD_SUB_PLAN,
       T02b.DESCR AS SUBPLAN_DESC,
       T03c.EXP_GRAD_TERM,
       T03c.DEGR_CHKOUT_STAT
FROM T03c
LEFT JOIN T01b ON T03c.ACAD_PROG = T01b.ACAD_PROG
LEFT JOIN T00b ON T03c.ACAD_PLAN = T00b.ACAD_PLAN
LEFT JOIN T02b ON T03c.ACAD_PLAN = T02b.ACAD_PLAN
    AND T03c.ACAD_SUB_PLAN = T02b.ACAD_SUB_PLAN;

--returns 94,858 rows
-- if this number doesn't match the number of rows from T03c
-- then we messed something up, since
-- all we are doing here is adding descriptions to T03c;

-- This will is our final all-in-one query which will match
-- the access job aid table "T04_CPPS data with setup tbl information"

